#include "const.h"
#include "object.h"
#include "screen.h"

#include <alloc.h>
#include <math.h>
#include <stdio.h>

char convertBSPPolys(BSP_Tree*);
void deleteBSPTree_with_BSPPolys(BSP_Tree* t);
void deleteBSPTree_with_TPolys(BSP_Tree* t);


/******************************************
draw is used to draw the object on (scr) from the (cam) viewpoint
******************************************/

void TObject::draw(TScreen* scr, TCam* cam, TVector* lght)
{
	int scr_width=scr->getwidth();
	int scr_height=scr->getheight();



	TVector light=*lght;

	//everything will get much simplier if we "put" the camera at (0,0,0), watching to (0,0,-1)

	//calculate the new centre
	TVector centre=pos;	//object's positioon

	//this puts the camera in the centre
	centre.x-=cam->pos.x;
	centre.y-=cam->pos.y;
	centre.z-=cam->pos.z;


	//rotate the centre by the camera angles in the negative direction
	//the same for the light source
	//this bizzare construct saves some time, because initialization
	//inside the rotateVectors(...) is time consuming and in this way
	//we rotate 2 vectors at once

	TQuat 	camquat=cam->quat;
	camquat.w=-camquat.w;


	TVector to_be_rotated[2];
	to_be_rotated[0]=centre;
	to_be_rotated[1]=light;

	rotateVectors(to_be_rotated, &camquat,2);

	centre=to_be_rotated[0];
	light=to_be_rotated[1];

	//determine whether is seen by the camera
	if (centre.z+radius<cam->dist/2)
	{
		float temp=(cam->dist-centre.z)/2/cam->dist;
		if (centre.x+radius > -temp*scr_width &&
			centre.x-radius < temp*scr_width &&
			centre.y+radius > -temp*scr_height &&
			centre.y-radius < temp*scr_height  )

		{
			/*********** The object is to be drawn ....
			 Now we determine, whether it will appear large enough
			 (diameter of at least MIN_PIXELS) to be drawn properly,
			 or whether we can just draw a little white square ...*/
			int diameter=(int)(radius*2*cam->dist/(cam->dist-centre.z));
			if (diameter<MIN_PIXELS)	//No! Draw a square
			{
				scr->setpen(shade(avgColour,128));
				int x=(int)(scr_width/2-(centre.x-cam->shiftRight)/(centre.z-cam->dist)*cam->dist-diameter/2+cam->shiftRight);
				int y=(int)(scr_height/2+centre.y/(centre.z-cam->dist)*cam->dist-diameter/2);
				scr->rectangle(x,y,x+diameter,y+diameter);

			}
			else	//Yes! Draw it properly
			{
				int i;	
				//copy the original's vertices
				for (i=0; i<vertCount;++i)
					vertices[i]=original->vertices[i];

				//rotate the object by its own angles and the camera angles - all at once
				//the object's centre is accumed to be at (0,0,0)

				TQuat composite;


				quatMult(&composite ,&camquat, &quat);

				rotateVectors(vertices, &composite, vertCount);

				//we now have the new centre and the object in the correct rotation
				//all we have to do is transpose the vertices from around (0,0,0)
				//to around the new centre
				//WE DO NOT WANT to transpose the normals, however! All the normals
				//are at the beginning of the array!
				for (i=normalCount; i<vertCount; ++i)
				{
					vertices[i].x+=centre.x;
					vertices[i].y+=centre.y;
					vertices[i].z+=centre.z;
				}

				//Project all the non-normal vertices on the "screen plane"
				//cam->dist is the eye<-->screen distance
				//the vertices2D array will hold the 2D coordinates x1,y1,x2,y2.....

				for (i=normalCount; i<vertCount; ++i)
				{
					//if (x>=0 && x<320 && y>=0 && y<200)
					vertices2D[2*i]=(int)(cam->dist*(vertices[i].x-cam->shiftRight)/(cam->dist-vertices[i].z)+scr_width/2+cam->shiftRight);
					vertices2D[2*i+1]=(int)(-cam->dist*vertices[i].y/(cam->dist-vertices[i].z)+scr_height/2);
				}

				//draw all the polygons - this is a recursive task because we need to
				//walk the BSP tree and thus we need a separate function for that
				drawPolygons(bspt, scr, cam, &light);
			}	//object is large enough
		}	//object seen
	}	//z<dist

}

/*********************
drawPolygons is a recursive function that walks the BSP tree (t)
and draws all the polygons in the correct order
*********************/
void TObject::drawPolygons(BSP_Tree* t, TScreen* scr, TCam* cam, TVector* light)
{
	if (!t) return;	//end of recursion

	int pos= (
		vertices[t->poly->normal].x * vertices[t->poly->vertices[1]].x+
		vertices[t->poly->normal].y * vertices[t->poly->vertices[1]].y+
		vertices[t->poly->normal].z * (vertices[t->poly->vertices[1]].z-cam->dist)
		> 0 );

	//is the camera (or better, the eye) on the positive or negative side? =>pos

	//walk the other way and draw the entire subtree
	if (pos) drawPolygons(t->pos, scr, cam, light);
		else drawPolygons(t->neg, scr, cam, light);

	//draw the polygon (t->poly)
	char side=t->poly->side;
	if (!side) side=pos?1:-1;
	if ((pos && side==1) || (!pos && side==-1))
	{
		//dir is the cosine of the angle between the normal and the light
		float dir=(vertices[t->poly->normal].x*light->x+
					vertices[t->poly->normal].y*light->y+
					vertices[t->poly->normal].z*light->z)*side;
		while (fabs(dir)>1) dir/=1.1;
		dir=acos(dir)/PI;
		int iDir=(int)(95+160*dir);	//96/255 ambient light

		scr->setpen(shade(t->poly->colour,iDir));
		scr->polygon(t->poly->vertCount, vertices2D, t->poly->vertices);
	}

	//walk the other subtree
	if (!pos) drawPolygons(t->pos, scr, cam, light);
		else drawPolygons(t->neg, scr, cam, light);

}


TObject::~TObject()
{
	if (!isOK()) return;
	if (vertices) delete vertices;

	if (!original)
	{
		deleteBSPTree_with_TPolys(bspt); //this is a template object! delete the bsp tree
		if (vertices2D) delete vertices2D;	//only template objects have this array!
	}
}
/********************************
TObject::TObject(TObject* orig)

This constructor creates a functional object using orig as a template.
The orig's vertices will copied in each calll to draw and the orig's
instance of BSP tree will be used
********************************/
TObject::TObject(TObject* orig)
{
	original=orig;		//save the template pointer

	//copy data
	ok=orig->ok;
	if (!ok) return;

	normalCount=orig->normalCount;
	vertCount=orig->vertCount;
	polyCount=orig->polyCount;
	pos=orig->pos;
	quat=orig->quat;
	bspt=orig->bspt;
	radius=orig->radius;
	avgColour=orig->avgColour;
	vertices2D=orig->vertices2D;
	//allocate the vertices array -
	//this has to be allocated for each instance, because it will be modified
	vertices=new TVector[vertCount];
	if (!vertices) {ok=0; return;}

}

/**********************************
TObject::TObject(char* s, int mag=1)

This object creates a non-functional template for other instances
Its main tasks are reading the vertices and polygons from the given file
and building the BSP tree - these will be available to the instances based
on this template. Mag can be used to scale the object being read. It defaults to 1.
***********************************/

TObject::TObject(char *s, float mag)
{
	FILE* f=fopen(s, "r");
	if (!f)	{ok=0; return;}
	/***********
	First, data is read from the file. BSP_Vector and BSP_Poly structures are
	used to hold the data. They're designed for the BSP algorithm.


	The input format is:
				//the vertices' coordinates
	x1	y1	z1
	x2	y2	z2
	..	..	..
	xn	yn	zn

	poly		//the word "poly" as an end-of-vertices-section mark

	N	colour	side v1	v2	v3	...	vN			//polygon definitions - v1 .. vN are vertex indexes, starting at 0
	N	colour	side v1	v2	v3	v4	...	vN

	blablabla	//anything non-numeric or EOF
	***********/

	//read the vertices into a linked list (vertList)
	//read x,y,z tripplets as long as possible
	ok=1;

	vertCount=0;
	BSP_Vector* vertList=0;

	float x,y,z;

	while (fscanf(f, "%f%f%f", &x, &y, &z) == 3)
	{
		BSP_Vector* tmp=new BSP_Vector;
			if (!tmp)	//alloc. err?
			{
				while (vertList) {tmp=vertList; vertList=vertList->next; delete tmp;}
				ok=0; 
				fclose(f); 
				return;
			}
		tmp->next=vertList;
		vertList=tmp;

		tmp->v.x=x*mag;
		tmp->v.y=y*mag;
		tmp->v.z=z*mag;
		++vertCount;
	}

	//read the word "poly" - the vertices-polygons delimiter
	
	fscanf(f, "poly");
	
	//read the polygons into a linked list

	BSP_Poly* polyList=0;
	BSP_Vector* newVertList=0;
		//new vertices will be added, but we cannot add them directly to vertList
		//because we need to assign the read-in vertices to the polygons
		//and we need the normals to be at the beginning of the list !!!!!

	//read as many polygons as possible
	int tmpvertcount;
	unsigned long avgRed=0, avgBlue=0, avgGreen=0;

	while (fscanf(f, "%d", &tmpvertcount) == 1)
	{
		BSP_Poly* tmp=new BSP_Poly;
			if (!tmp) {ok=0; break;}

		tmp->next=polyList;
		polyList=tmp;


		tmp->vertCount=tmpvertcount;

		unsigned long _col;
		unsigned int _side;
		
		if (fscanf(f, "%lx%d", &_col, &_side) != 2)
			{ok=0; break;}
		tmp->colour=_col;
		tmp->side=_side;

		tmp->vertices=new BSP_Vector* [tmp->vertCount];
			//allocate space for vertex pointers
			if (!tmp) {ok=0; break;}

		//assign the correct vertices to the polygon
		for (int j=0; j<tmp->vertCount; ++j)
		{

			int vertex;
			if (fscanf(f, "%d", &vertex) != 1)
				{ok=0; break;}
					
			BSP_Vector* v=vertList;
			for (int i=0; i<vertCount-vertex-1; ++i)
				v=v->next;
				//the linked list is reversed!

			tmp->vertices[j]=v;
		}
		if (!ok) break;


		//calculate the polygon's normal
		TVector A,B,C;
		A=tmp->vertices[0]->v;
		B=tmp->vertices[1]->v;
		C=tmp->vertices[2]->v;
		A.x-=B.x;
		A.y-=B.y;
		A.z-=B.z;

		C.x-=B.x;
		C.y-=B.y;
		C.z-=B.z;

		B.x = (A.y * C.z) - (C.y * A.z);
		B.y = (A.z * C.x) - (C.z * A.x);
		B.z = (A.x * C.y) - (C.x * A.y);

		float root=sqrt((float)B.x*B.x + (float)B.y*B.y + (float)B.z*B.z);
		B.x/=root; B.y/=root; B.z/=root;

		//add the normal to the newVertList (which will LATER be added to vertList)
		tmp->normal=new BSP_Vector;
			if (!tmp) {ok=0; break;}
		tmp->normal->next=newVertList;
		newVertList=tmp->normal;
		tmp->normal->v=B;

		//calculate the polygon's plane equation constant
		A=tmp->vertices[0]->v;
		tmp->d=-(A.x*B.x + A.y*B.y + A.z*B.z);

		//average colour...
		avgRed+=tmp->colour&clRed;
		avgGreen+=tmp->colour&clGreen;
		avgBlue+=tmp->colour&clBlue;

	}
	fclose(f);
	//clean-up in case of alloc. error
	if (!ok)
	{
		while (newVertList) {BSP_Vector* tmp=newVertList; newVertList=newVertList->next; delete tmp;}
		while (vertList) {BSP_Vector* tmp=vertList; vertList=vertList->next; delete tmp;}
		while (polyList)
		{
			BSP_Poly* tmp=polyList; polyList=polyList->next;
			delete tmp->vertices; delete tmp;
		}
		return;
	}

	/***********************************************/
	//build the object's BSP tree

	polyCount=0;	//will be updated by buildBSPTree

	if (buildBSPTree(&bspt , &polyList, &vertList))  //err?
		ok=0;

	avgRed/=polyCount;
	avgGreen/=polyCount;
	avgBlue/=polyCount;

	avgColour=(avgRed&clRed)|(avgGreen&clGreen)|(avgBlue&clBlue);


	//add newVertList to vertList
	//Normals must go to the beginning of the list!!!
	normalCount=0;
	while (newVertList)
	{
		BSP_Vector* tmp=vertList;

		vertList=newVertList;
		newVertList=newVertList->next;
		vertList->next=tmp;

		++vertCount;
		++normalCount;
	}
	/***********************************************/
	//convert the dynamic linked list (vertList) into an array

	vertices=new TVector[vertCount];	//allocate the vertices[] array
		if (!vertices) ok=0;

	if (ok)
	{
		BSP_Vector *tmp=vertList;
		//copy the corresponding vertices into the array
		for (int i=0;i<vertCount; ++i)
		{
			vertices[i]=tmp->v;
			tmp->index=i;
				//save the original index, so that the polygons (that only have
				//pointers to their vertices) can use the index
			tmp=tmp->next;
		}

		//convert the BSP_Polys in the BSP tree into TPolys
		//the main difference is that the TPolygons' vertices are array indexes, not pointers!
		if(convertBSPPolys (bspt)) ok=0;
	}

	vertices2D=new int[2*vertCount];
	if (!vertices2D) ok=0;


	if (!ok)
	//clean-up in case of alloc. error
	{
		deleteBSPTree_with_BSPPolys(bspt);	//remove the tree

		while (polyList)	//destroy the polyList
		{
			BSP_Poly* tmp=polyList; polyList=polyList->next;
			delete tmp->vertices; delete tmp;
		}
		return;
	}

	//destroy the vertList
	while (vertList)
	{
		BSP_Vector* tmp=vertList->next;
		delete vertList;
		vertList=tmp;
	}

	if (!ok) return;

	original=0;
		//this can be used to determine if the object is just a template
		//or a functional TObject

	//calculate the average vertex length and store it in (radius)

	radius=0;
	for (int i=normalCount; i<vertCount; ++i)
		radius+=sqrt(vertices[i].x*vertices[i].x + vertices[i].y*vertices[i].y + vertices[i].z*vertices[i].z);

	radius/=vertCount-normalCount;


}


/*********************
buildBSPTree's functionality belongs to the constructor. It is, however
a recursive function and thus needs to be separate. Because polygons need
to be deleted from the list and new vertices may need to be addedd,
polyList and vertList linked lists are passed by reference. tree is the
pointer to the node we should start at. (the root when invoked from
the constructor)
**********************/

char TObject::buildBSPTree (BSP_Tree** tree, BSP_Poly** polyList, BSP_Vector** vertList)
{
	//get the polygon that will cause the least splits
	int minSplits=9999;
	BSP_Poly* bestPoly;

	BSP_Poly* p;
	for(p=*polyList; p; p=p->next)	//for all the polygons
	{
		int splits=0;
			//the number of splits p would cause if it were used as the split-plane
		for(BSP_Poly* q=*polyList; q; q=q->next) if (p!=q)
			//for all the polygons, other than p
		{
			int sides=0;
			for (int i=0; i< q->vertCount; ++i)	//for all of q's vertices
			{
				float side=
					+q->vertices[i]->v.x * p->normal->v.x
					+q->vertices[i]->v.y * p->normal->v.y
					+q->vertices[i]->v.z * p->normal->v.z
					+p->d;

				if (fabs (side) > 0.01)		//prevent rounding error
					sides|=(side>0 ? 1:2);
						//bits 1/2 in (sides) mean:
						//	q has a vertex on the positive/negative side of p
			}

			if (sides==3) ++splits;	//if q's vertices are on different sides of p
		}

		//so, is p the best one so far?
		if (splits<minSplits)
		{
			minSplits=splits;
			bestPoly=p;
		}
	}

	//take bestPoly out of the polyList
	BSP_Poly ** _tmp=polyList;
	while (*_tmp!=bestPoly)	_tmp=&((*_tmp)->next);
	*_tmp=(*_tmp)->next;

	//create a new tree node with bestPoly
	*tree=new BSP_Tree;
	if (!*tree) return 1;

	(*tree)->bsppoly=bestPoly;
	(*tree)->pos=(*tree)->neg=0;	//no children yet
	++polyCount;	//one more polygon in the tree


	//OK, we have the split-plane....
	//Now, take all the polygons in polyList and put them in
	//posList (negList), if they're on the positive (negative) side of bestPoly
	//or split them, if the are on both sides, and put the newly created
	//polygons in the appropriate lists

	BSP_Poly *posList=0, *negList=0;	//empty pos and neg lists
	int posCount=0, negCount=0;

	while (*polyList)	//for each polygon
		//we'll always work with the first polygon in polyList
		//and will tear it of in the end of the loop
	{

		/******
		GET THE SIDE FOR EACH VERTEX
		first, take each vertex of **polyList and test which side of bestPoly
		it is on. We can use the index variable to temporarily store the result
		******/
		int poscnt=0, negcnt=0, zerocnt=0;
			//we will count the vertices that are positive, negative and
			//(they can as well) the ones that lie on bestPoly

		for (int i=0; i< (*polyList)->vertCount; ++i)
		{
			
			float side=
				+bestPoly->normal->v.x * (*polyList)->vertices[i]->v.x
				+bestPoly->normal->v.y * (*polyList)->vertices[i]->v.y
				+bestPoly->normal->v.z * (*polyList)->vertices[i]->v.z
				+bestPoly->d;

			int iside;
			if (fabs (side) < 0.01) {iside=0; ++zerocnt;}	//prevent rounding error
				else if (side>0) {iside=1; ++poscnt;}
					else {iside=-1; ++negcnt;}

			(*polyList)->vertices[i]->index=iside;	//store the side in the vertex.index
		}


		/********
		SPLIT
		If the polygon needs to be split, up to two new vertices have to
		be added (new1 and new2). A vertex has to be added if one edge starts
		on the positive side of bestPoly and ends on the negative side.
		********/
		BSP_Vector *new1=0, *new2=0;
		if (poscnt && negcnt)
			//only do this when the polygon has a vertex on both sides
		{

			BSP_Vector *A,*B;
			//take A-B edges in turn....
			A=(*polyList)->vertices[(*polyList)->vertCount-1];
			//start with A being the last vertex and B the first one
			for (int i=0; i<(*polyList)->vertCount; ++i)
			{
				B=(*polyList)->vertices[i];
				if (B->index==-A->index)
					//if A and B are on different sides
				{

					//add a vertex to vertList
					BSP_Vector* tmp=new BSP_Vector;
						if (!tmp)	//err?
						{
							if (new1) delete new1;
							return 1;
						}
					tmp->next=(*vertList);
					(*vertList)=tmp;

					//save the new vertex's address in new1, or new2 if new1 is full
					if (new1) new2=(*vertList);
						else new1=(*vertList);

					//new vertex has been added
					++vertCount;

					//a bit of geometry ... calculate the intersection of
					//the A-B edge with bestPoly
					float t=  -(
						bestPoly->normal->v.x * A->v.x+
						bestPoly->normal->v.y * A->v.y+
						bestPoly->normal->v.z * A->v.z+
						bestPoly->d
						) / (
						bestPoly->normal->v.x * (B->v.x - A->v.x)+
						bestPoly->normal->v.y * (B->v.y - A->v.y)+
						bestPoly->normal->v.z * (B->v.z - A->v.z)
						);

					(*vertList)->v.x=A->v.x+t*(B->v.x-A->v.x);
					(*vertList)->v.y=A->v.y+t*(B->v.y-A->v.y);
					(*vertList)->v.z=A->v.z+t*(B->v.z-A->v.z);
					(*vertList)->index=i;
						//save the i ... will be used to put the new
						//vertex in the right place

					//we added a vertex which is on bestPoly
					++zerocnt;
				}

				A=B;	//in next iteration, A is what used to be B
			}

		}

		/********
		We now have the number of vertices the poly has on the +side, -side
		and the ones that lie on bestPoly (0side). Take all the ones on the
		+side and 0side	and make a +polygon out of them .... the same with -
		********/

		BSP_Poly *pos=0, *neg=0;

		//any vertices on the +side
		if (poscnt)
		{
			pos=new BSP_Poly;	//a new polygon in the posList
			if (pos)
			{
				memcpy(pos, (*polyList), sizeof(BSP_Poly));
					//copy everything from the old polygon

				pos->vertCount=poscnt+zerocnt;	//all vertices on +side and 0side
				pos->vertices=new BSP_Vector* [pos->vertCount];	//new array

				int j=0;	//the new polygon's vertex index
				//for all the vertices of the old polygon....
				if (pos->vertices) for (int i=0; i<(*polyList)->vertCount; ++i)
				{
					//is there a new1/new2 and should it be inserted into
					//the new polygon's vertices now?
					if (new1 && new1->index==i) pos->vertices[j++]=new1;
					if (new2 && new2->index==i) pos->vertices[j++]=new2;

					//is the i'th old-poly's vertex on our side?
					if ((*polyList)->vertices[i]->index >=0)
						pos->vertices[j++]=(*polyList)->vertices[i];
				} else ok=0;

				//add the new polygon to the posList
				pos->next=posList;
				posList=pos;
				++posCount;
			} else ok=0;
		}


		//The same as above for the negative side...
		//Also add polygons that are entirely in the plane of bestPoly
		if (negcnt || (!poscnt && zerocnt))
		{
			neg=new BSP_Poly;
			if (neg)
			{
				memcpy(neg, (*polyList), sizeof(BSP_Poly));

				neg->vertCount=negcnt+zerocnt;
				neg->vertices=new BSP_Vector * [neg->vertCount];


				int j=0;
				if (neg->vertices) for (int i=0; i<(*polyList)->vertCount; ++i)
				{
					if (new1 && new1->index==i) neg->vertices[j++]=new1;
					if (new2 && new2->index==i) neg->vertices[j++]=new2;
					if ((*polyList)->vertices[i]->index <=0)
						neg->vertices[j++]=(*polyList)->vertices[i];
				} else ok=0;

				neg->next=negList;
				negList=neg;
				++negCount;
			} else ok=0;
		}


		//delete the old polygon
		delete (*polyList)->vertices;
		BSP_Poly* tmp=(*polyList)->next;
		delete (*polyList);
		(*polyList)=tmp;

		if (!ok)
		{
			BSP_Poly *t;
			while (negList) {t=negList; negList=negList->next; if (t->vertices) delete t->vertices; delete t;}
			while (posList) {t=posList; posList=posList->next; if (t->vertices) delete t->vertices; delete t;}
			return 1;
		}


	}



	//OK, call recursively ... pass the posList (negList) as the polygon list
	char err=0;
	if (posList)         err|=buildBSPTree (&(*tree)->pos, &posList, vertList);
	if (negList && !err) err|=buildBSPTree (&(*tree)->neg, &negList, vertList);
	return err;
}


/******************************************
convertBSPPolys is used to recursively change the BSP_Poly structures
in a newly built BSP tree into TPoly structures
*******************************************/
char convertBSPPolys(BSP_Tree* t)
{
	//because sizeof(TPoly)<sizeof(BSP_POLY) && sizeof(int)<sizeof(BSP_Vector*),
	//...an allocation error can only occur the first time this function is called
	//   and not in the recursive calls

	TPolygon * newpoly=new TPolygon;		//allocate the new TPoly
		if (!newpoly) return 1;
	//the important difference between a BSP_Poly and a TPoly is, that
	//vertices TPoly vertices are given by array indexes, not pointers
	BSP_Poly *tmp=t->bsppoly;	//!!!! bsppoly and poly are union members!!!
	newpoly->vertCount=tmp->vertCount;	//copy the data values
	newpoly->colour=tmp->colour;
	newpoly->side=tmp->side;

	newpoly->vertices=new int[newpoly->vertCount]; //allocate the vertices array
	if (!newpoly->vertices)
	{
		delete newpoly;
		return 1;
	}

	t->poly=newpoly;

	//this assumes that the vertices have their index set to their actual
	//array index - done while transforming the BSP_Vector list into a TVector array

	for (int j=0; j<t->poly->vertCount; ++j)
		t->poly->vertices[j]=tmp->vertices[j]->index;

	t->poly->normal=tmp->normal->index;	//the same for the normal

	//destroy the old structure
	delete tmp->vertices;
	delete tmp;

	//process both subtrees recursively
	if (t->pos) convertBSPPolys(t->pos);
	if (t->neg) convertBSPPolys(t->neg);

	return 0;
}

/******************************************
deleteBSPTree is used to recursively de-allocate the tree
2 versions are necessary:
one to cleanup if an error occurs before conversion to TPoly
and one to deallocate a converted tree in the destructor
*******************************************/
void deleteBSPTree_with_BSPPolys(BSP_Tree* t)
{
	if (t)
	{
		if (t->bsppoly->vertices) delete t->bsppoly->vertices;
		delete t->bsppoly;

		//process both subtrees recursively
		deleteBSPTree_with_BSPPolys(t->pos);
		deleteBSPTree_with_BSPPolys(t->neg);
	}
}


void deleteBSPTree_with_TPolys(BSP_Tree* t)
{
	if (t)
	{
		if (t->poly->vertices) delete t->poly->vertices;
		delete t->poly;

		//process both subtrees recursively
		deleteBSPTree_with_TPolys(t->pos);
		deleteBSPTree_with_TPolys(t->neg);
	}
}

/* dump() and read() are complementary.
 * Dump stores the instance-specific variables in a buffer network
 * cloning.
*/

char* TObject::dump(char* buff)
{
	signed int *b=(int*)buff;
	*b++ = PACK(pos.x, 256);
	*b++ = PACK(pos.y, 256);
	*b++ = PACK(pos.z, 256);


	*b++ = PACK(quat.x, 1E9);
	*b++ = PACK(quat.y, 1E9);
	*b++ = PACK(quat.z, 1E9);
	*b++ = PACK(quat.w, 1E9);

	return (char*)b;	
}

char* TObject::read(char* buff)
{
	signed int *b=(int*)buff;

	pos.x = UNPACK(*b++, 256);
	pos.y = UNPACK(*b++, 256);
	pos.z = UNPACK(*b++, 256);


	quat.x = UNPACK(*b++, 1E9);
	quat.y = UNPACK(*b++, 1E9);
	quat.z = UNPACK(*b++, 1E9);
	quat.w = UNPACK(*b++, 1E9);

	return (char*)b;
}
