#ifndef __SHIP
#define __SHIP

//flying object types
#define TYPE_SHIP 0
#define TYPE_MISSILE 1
#define TYPE_EXPLOSION 2
#define TYPE_SHOT 3

#define TYPE_SHELLS 4
#define TYPE_SHIELDS 5
#define TYPE_MISSILES 6

#define TYPE_NOTHING 99

class TFlyingObject;
class TMissile;
class TShip;

#include "space.h"
#include "object.h"
#include "const.h"

struct TController	//used for human-controlled ships
{
	float up, left, forward;	//user input
	struct __flags
	{
		char cannon:1;
		char missile:1;
	} flags;

	long locked;        //the ID of the locked ship or 0 - used for drawing to the screen
};
//********************* TFlyingObject ****************************
//base class for objects flying in TSpace
class TFlyingObject: public TObject
{
public:

	TFlyingObject(TObject*, TSpace* spc, int _owner);
	virtual ~TFlyingObject();

	TVector getVVect();	//curent speed vector

	inline float getV() {return v;}
	inline void setV(float _v) {v=_v;}
	inline int getOwner() {return owner;}
	inline int getLives() {return lives;}

	//moves an object, given the change in acceleration for each direction and the time unit
	void move(float da_up, float da_left, float da_forward, float dt);

	virtual char tick(float dt){return 0;};		//sub-class specific behaviour

	virtual void collide(TFlyingObject* o){};	//called by when two objects collide (o is the other object)
	virtual int collisionPower(){return 0;};	//returns the damaging power
	virtual char type() {return 0;};				//returns a sub-class specific type

	virtual void die() {lives=-1;};							//should be called to be removed from the space properly
	void explode(int size);						//initiate an explosion

	virtual char* dump(char *buff);	//dumps all the instance-specific data into buff for network-cloning
	virtual char* read(char *buff);  //reads all the instance-specific data from buff


protected:
	TSpace* space;		//the space we're in
	TVector a;          //current acceleration
	float v;			//current velocity (direction given by quat)
	float maxspeed;		//maximum velocity
	int owner;			//used for score-counting - gets passed from ships to miisiles and shots
	int lives;			//health. If negative, the space will destroy us
};

//************************ TShip ************
class TShip: public TFlyingObject
{
public:
	TShip(TObject*, TSpace* s, int owner);

	void lockedTick(TFlyingObject* locked, float dt);	//follow a given ship

	virtual char tick(float dt);
	virtual void collide(TFlyingObject* o);
	virtual char type() {return TYPE_SHIP;};
	virtual int collisionPower() {return SHIP_CPOWER;}
	void cannon();		//if possible, fire a cannon shell (shot)
	void missile();		//if possible, launch a missile

	int getShells() {return shells;}
	int getMissiles() {return missiles;}

	virtual char* dump(char *buff);	//dumps all the instance-specific data into buff for network-cloning
	virtual char* read(char *buff);  //reads all the instance-specific data from buff


	TController* controller;	//0 for computer controlled ship. Otherwise a pointer to the controlling TController

	int lockedLives;
	int lockedOwner;
private:
	long findRandomObject(int type);	//find a random object of the given type

	int shells, missiles;	//number of available shells/missiles

	long lockedID;		//ID of the ship we follow
	float lastdist;		// used in lockedTick(...)

	float lastShot;		//time interval since the last shot was fired
	float lastMissile;  //time interval since the last missile was launched

	float lastdt;	    //used in cannon()
};

//*********************** TMissile **********************
//a guided missile
class TMissile: public TFlyingObject
{
public:
	TMissile(TSpace* s, int owner, long lock);
	void lockedTick(TFlyingObject* locked, float dt);
	virtual char tick(float dt);
	virtual void collide(TFlyingObject* o);
	virtual char type() {return TYPE_MISSILE;};
	virtual int collisionPower() {return MISSILE_CPOWER;}

private:
	long lockedID;	//ID of the ship we were fired at
	float lastdist;	//used in lockedTick(...)
	long ttl;
};

//*********************** TShot **********************
//a fired cannon shell flying through space

class TShot: public TFlyingObject
{
public:
	TShot(TSpace* s, int owner);
	virtual char tick(float dt);
	virtual void collide(TFlyingObject* o);
	virtual char type() {return TYPE_SHOT;};
	virtual int collisionPower() {return SHOT_CPOWER;}

private:
	float ttl;	//will die after this much time has elapsed
};


//*********************** TExplosion **********************
//a piece of garbage from an exploded object
class TExplosion: public TFlyingObject
{
public:
	TExplosion(TVector _pos,  TSpace* s, int _size);
	virtual char tick(float dt);
	virtual void collide(TFlyingObject* o);
	virtual char type() {return TYPE_EXPLOSION;};
	virtual int collisionPower() {return 0;}

private:
	int size;	//will die when decreased to 0
	TVector V;	//the velocity
	TQuat Q;	//rotation
};

//********************** GOODIES flying through space **********
class TShells: public TFlyingObject
{
public:
	TShells(TVector _pos,  TSpace* s);
	virtual char tick(float) {return 0;};
	virtual void collide(TFlyingObject*) {explode(5);die();}
	virtual char type() {return TYPE_SHELLS;}
	virtual int collisionPower() {return 1;}
};

class TShields: public TFlyingObject
{
public:
	TShields(TVector _pos,  TSpace* s);
	virtual char tick(float) {return 0;};
	virtual void collide(TFlyingObject*) {explode(5);die();}
	virtual char type() {return TYPE_SHIELDS;}
	virtual int collisionPower() {return 1;}
};

class TMissiles: public TFlyingObject
{
public:
	TMissiles(TVector _pos,  TSpace* s);
	virtual char tick(float) {return 0;};
	virtual void collide(TFlyingObject* ) {explode(15);die();}
	virtual char type() {return TYPE_MISSILES;}
	virtual int collisionPower() {return 1;}
};

#endif

